@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -20,6 +20,7 @@
 #include "config.h"
 
 #import "OFMutableIRI.h"
+#import "OFIRI.h"
 #import "OFIRI+Private.h"
 #import "OFArray.h"
 #import "OFDictionary.h"
@@ -32,6 +33,7 @@
 
 #import "OFInvalidArgumentException.h"
 #import "OFInvalidFormatException.h"
+#import "OFOutOfRangeException.h"
 
 @implementation OFMutableIRI
 @dynamic scheme, host, percentEncodedHost, port, user, percentEncodedUser;
@@ -46,7 +48,7 @@
 
 - (instancetype)initWithScheme: (OFString *)scheme
 {
-	self = [super of_init];
+	self = [self of_init];
 
 	@try {
 		self.scheme = scheme;
@@ -67,7 +69,7 @@
 	if (scheme.length < 1 || !OFASCIIIsAlpha(*scheme.UTF8String))
 		@throw [OFInvalidFormatException exception];
 
-	OFIRIVerifyIsEscaped(scheme,
+	_OFIRIVerifyIsEscaped(scheme,
 	    [OFCharacterSet IRISchemeAllowedCharacterSet], false);
 
 	_scheme = [scheme.lowercaseString copy];
@@ -82,7 +84,7 @@
 	void *pool = objc_autoreleasePoolPush();
 	OFString *old = _percentEncodedHost;
 
-	if (OFIRIIsIPv6Host(host))
+	if (_OFIRIIsIPv6Host(host))
 		_percentEncodedHost = [[OFString alloc]
 		    initWithFormat: @"[%@]", host];
 	else
@@ -101,11 +103,11 @@
 
 	if ([percentEncodedHost hasPrefix: @"["] &&
 	    [percentEncodedHost hasSuffix: @"]"]) {
-		if (!OFIRIIsIPv6Host([percentEncodedHost substringWithRange:
+		if (!_OFIRIIsIPv6Host([percentEncodedHost substringWithRange:
 		    OFMakeRange(1, percentEncodedHost.length - 2)]))
 			@throw [OFInvalidFormatException exception];
 	} else if (percentEncodedHost != nil)
-		OFIRIVerifyIsEscaped(percentEncodedHost,
+		_OFIRIVerifyIsEscaped(percentEncodedHost,
 		    [OFCharacterSet IRIHostAllowedCharacterSet], true);
 
 	old = _percentEncodedHost;
@@ -117,8 +119,17 @@
 {
 	OFNumber *old = _port;
 
-	if (port.longLongValue < 0 || port.longLongValue > 65535)
+	@try {
+#if USHRT_MAX == 65535
+		/* Range check */
+		(void)port.unsignedShortValue;
+#else
+		if (port.unsignedShortValue > 65535)
+			@throw [OFInvalidArgumentException exception];
+#endif
+	} @catch (OFOutOfRangeException *e) {
 		@throw [OFInvalidArgumentException exception];
+	}
 
 	_port = [port copy];
 	[old release];
@@ -143,7 +154,7 @@
 	OFString *old;
 
 	if (percentEncodedUser != nil)
-		OFIRIVerifyIsEscaped(percentEncodedUser,
+		_OFIRIVerifyIsEscaped(percentEncodedUser,
 		    [OFCharacterSet IRIUserAllowedCharacterSet], true);
 
 	old = _percentEncodedUser;
@@ -170,7 +181,7 @@
 	OFString *old;
 
 	if (percentEncodedPassword != nil)
-		OFIRIVerifyIsEscaped(percentEncodedPassword,
+		_OFIRIVerifyIsEscaped(percentEncodedPassword,
 		    [OFCharacterSet IRIPasswordAllowedCharacterSet], true);
 
 	old = _percentEncodedPassword;
@@ -196,7 +207,7 @@
 {
 	OFString *old;
 
-	OFIRIVerifyIsEscaped(percentEncodedPath,
+	_OFIRIVerifyIsEscaped(percentEncodedPath,
 	    [OFCharacterSet IRIPathAllowedCharacterSet], true);
 
 	old = _percentEncodedPath;
@@ -242,7 +253,7 @@
 	OFString *old;
 
 	if (percentEncodedQuery != nil)
-		OFIRIVerifyIsEscaped(percentEncodedQuery,
+		_OFIRIVerifyIsEscaped(percentEncodedQuery,
 		    [OFCharacterSet IRIQueryAllowedCharacterSet], true);
 
 	old = _percentEncodedQuery;
@@ -309,7 +320,7 @@
 	OFString *old;
 
 	if (percentEncodedFragment != nil)
-		OFIRIVerifyIsEscaped(percentEncodedFragment,
+		_OFIRIVerifyIsEscaped(percentEncodedFragment,
 		    [OFCharacterSet IRIFragmentAllowedCharacterSet], true);
 
 	old = _percentEncodedFragment;
