@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -33,8 +33,8 @@ OF_ASSUME_NONNULL_BEGIN
 #ifdef __cplusplus
 extern "C" {
 #endif
-extern size_t OFStrFTime(char *buffer, size_t bufferLen, const char *format,
-    struct tm *tm, short tz);
+extern size_t _OFStrFTime(char *buffer, size_t bufferLen, const char *format,
+    struct tm *tm, short tz) OF_VISIBILITY_HIDDEN;
 #ifdef __cplusplus
 }
 #endif
