@@ -23,6 +23,9 @@
 #import "OHN64Controller.h"
 #import "OHN64Controller+Private.h"
 #import "OFDictionary.h"
+#ifdef OF_HAVE_GCF
+# import "OFString+NSObject.h"
+#endif
 #import "OHGameControllerButton.h"
 #import "OHGameControllerElement.h"
 #import "OHGameControllerElement+Private.h"
@@ -36,7 +39,43 @@ static OFString *const buttonNames[] = {
 };
 static const size_t numButtons = sizeof(buttonNames) / sizeof(*buttonNames);
 
+#ifdef OF_HAVE_GCF
+static OFDictionary<OFString *, NSString *> *buttonsMap;
+static OFDictionary<OFString *, NSString *> *directionalPadsMap;
+#endif
+
 @implementation OHExtendedN64Controller
+#ifdef OF_HAVE_GCF
++ (void)initialize
+{
+	void *pool;
+
+	if (self != [OHExtendedN64Controller class])
+		return;
+
+	pool = objc_autoreleasePoolPush();
+
+	buttonsMap = [[OFDictionary alloc] initWithKeysAndObjects:
+	    @"A", @"Button A".NSObject,
+	    @"B", @"Button B".NSObject,
+	    @"L", @"Left Shoulder".NSObject,
+	    @"R", @"Right Shoulder".NSObject,
+	    @"Z", @"Left Trigger".NSObject,
+	    @"Start", @"Button Menu".NSObject,
+	    @"ZR", @"Right Trigger".NSObject,
+	    @"Home", @"Button Home".NSObject,
+	    @"Capture", @"Button Share".NSObject,
+	    nil];
+	directionalPadsMap = [[OFDictionary alloc] initWithKeysAndObjects:
+	    @"Thumbstick", @"Left Thumbstick".NSObject,
+	    @"C-Buttons", @"Right Thumbstick".NSObject,
+	    @"D-Pad", @"Direction Pad".NSObject,
+	    nil];
+
+	objc_autoreleasePoolPop(pool);
+}
+#endif
+
 - (instancetype)oh_init
 {
 	self = [super oh_init];
@@ -54,7 +93,7 @@ static const size_t numButtons = sizeof(buttonNames) / sizeof(*buttonNames);
 		}
 		[buttons makeImmutable];
 		[_buttons release];
-		_buttons = [buttons retain];
+		_buttons = [buttons copy];
 
 		objc_autoreleasePoolPop(pool);
 	} @catch (id e) {
@@ -80,4 +119,21 @@ static const size_t numButtons = sizeof(buttonNames) / sizeof(*buttonNames);
 	return [super oh_buttonForEvdevButton: button];
 }
 #endif
+
+#ifdef OF_HAVE_GCF
+- (OFDictionary<OFString *, NSString *> *)oh_buttonsMap
+{
+	return buttonsMap;
+}
+
+- (OFDictionary<OFString *, NSString *> *)oh_axesMap
+{
+	return [OFDictionary dictionary];
+}
+
+- (OFDictionary<OFString *, NSString *> *)oh_directionalPadsMap
+{
+	return directionalPadsMap;
+}
+#endif
 @end
