@@ -57,7 +57,9 @@ static const size_t numButtons = sizeof(buttonNames) / sizeof(*buttonNames);
 		OFMutableDictionary *directionalPads;
 		OHGameControllerAxis *xAxis, *yAxis;
 		OHGameControllerDirectionalPad *directionalPad;
+#ifndef OF_HAVE_GCF
 		OHGameControllerButton *up, *down, *left, *right;
+#endif
 
 		for (size_t i = 0; i < numButtons; i++) {
 			OHGameControllerButton *button = [OHGameControllerButton
@@ -66,15 +68,17 @@ static const size_t numButtons = sizeof(buttonNames) / sizeof(*buttonNames);
 			[buttons setObject: button forKey: buttonNames[i]];
 		}
 		[buttons makeImmutable];
-		_buttons = [buttons retain];
+		_buttons = [buttons copy];
 
 		directionalPads =
 		    [OFMutableDictionary dictionaryWithCapacity: 3];
 
-		xAxis = [OHGameControllerAxis oh_elementWithName: @"X"
-							  analog: true];
-		yAxis = [OHGameControllerAxis oh_elementWithName: @"Y"
-							  analog: true];
+		xAxis = [OHGameControllerAxis
+		    oh_elementWithName: @"Thumbstick X"
+				analog: true];
+		yAxis = [OHGameControllerAxis
+		    oh_elementWithName: @"Thumbstick Y"
+				analog: true];
 		directionalPad = [OHGameControllerDirectionalPad
 		    oh_padWithName: @"Thumbstick"
 			     xAxis: xAxis
@@ -94,6 +98,17 @@ static const size_t numButtons = sizeof(buttonNames) / sizeof(*buttonNames);
 			    analog: false];
 		[directionalPads setObject: directionalPad forKey: @"D-Pad"];
 
+#ifdef OF_HAVE_GCF
+		xAxis = [OHGameControllerAxis oh_elementWithName: @"C-Buttons X"
+							  analog: false];
+		yAxis = [OHGameControllerAxis oh_elementWithName: @"C-Buttons Y"
+							  analog: false];
+		directionalPad = [OHGameControllerDirectionalPad
+		    oh_padWithName: @"C-Buttons"
+			     xAxis: xAxis
+			     yAxis: yAxis
+			    analog: false];
+#else
 		up = [OHGameControllerButton oh_elementWithName: @"C-Up"
 							 analog: false];
 		down = [OHGameControllerButton oh_elementWithName: @"C-Down"
@@ -109,11 +124,12 @@ static const size_t numButtons = sizeof(buttonNames) / sizeof(*buttonNames);
 			      left: left
 			     right: right
 			    analog: false];
+#endif
 		[directionalPads setObject: directionalPad
 				    forKey: @"C-Buttons"];
 
 		[directionalPads makeImmutable];
-		_directionalPads = [directionalPads retain];
+		_directionalPads = [directionalPads copy];
 
 		objc_autoreleasePoolPop(pool);
 	} @catch (id e) {
