@@ -22,7 +22,11 @@
 #import "OHRightJoyCon.h"
 #import "OHRightJoyCon+Private.h"
 #import "OFDictionary.h"
+#ifdef OF_HAVE_GCF
+# import "OFString+NSObject.h"
+#endif
 #import "OHGameControllerAxis.h"
+#import "OHGameControllerAxis+Private.h"
 #import "OHGameControllerButton.h"
 #import "OHGameControllerDirectionalPad.h"
 #import "OHGameControllerDirectionalPad+Private.h"
@@ -35,14 +39,49 @@
 #endif
 
 static OFString *const buttonNames[] = {
-	@"X", @"B", @"A", @"Y", @"R", @"ZR", @"Right Thumbstick", @"+",
-	@"Home", @"SL", @"SR"
+	@"X", @"B", @"A", @"Y", @"SL", @"SR", @"+",
+#ifndef OF_HAVE_GCF
+	/* GameController.framework doesn't expose a lot of buttons. */
+	@"R", @"ZR", @"Left Thumbstick", @"Home"
+#endif
 };
 static const size_t numButtons = sizeof(buttonNames) / sizeof(*buttonNames);
 
+#ifdef OF_HAVE_GCF
+static OFDictionary<OFString *, NSString *> *buttonsMap;
+static OFDictionary<OFString *, NSString *> *directionalPadsMap;
+#endif
+
 @implementation OHRightJoyCon
 @synthesize buttons = _buttons, directionalPads = _directionalPads;
 
+#ifdef OF_HAVE_GCF
++ (void)initialize
+{
+	void *pool;
+
+	if (self != [OHRightJoyCon class])
+		return;
+
+	pool = objc_autoreleasePoolPush();
+
+	buttonsMap = [[OFDictionary alloc] initWithKeysAndObjects:
+	    @"A", @"Button A".NSObject,
+	    @"B", @"Button B".NSObject,
+	    @"X", @"Button X".NSObject,
+	    @"Y", @"Button Y".NSObject,
+	    @"SL", @"Left Shoulder".NSObject,
+	    @"SR", @"Right Shoulder".NSObject,
+	    @"+", @"Button Menu".NSObject,
+	    nil];
+	directionalPadsMap = [[OFDictionary alloc] initWithKeysAndObjects:
+	    @"Left Thumbstick", @"Direction Pad".NSObject,
+	    nil];
+
+	objc_autoreleasePoolPop(pool);
+}
+#endif
+
 - (instancetype)init
 {
 	OF_INVALID_INIT_METHOD
@@ -66,21 +105,26 @@ static const size_t numButtons = sizeof(buttonNames) / sizeof(*buttonNames);
 			[buttons setObject: button forKey: buttonNames[i]];
 		}
 		[buttons makeImmutable];
-		_buttons = [buttons retain];
+		_buttons = [buttons copy];
 
-		xAxis = [OHGameControllerAxis oh_elementWithName: @"X"
-							  analog: true];
-		yAxis = [OHGameControllerAxis oh_elementWithName: @"Y"
-							  analog: true];
+		xAxis = [OHGameControllerAxis
+		    oh_elementWithName: @"Left Thumbstick X"
+				analog: true];
+#if defined(OF_LINUX) && defined(OF_HAVE_FILES)
+		xAxis.oh_inverted = true;
+#endif
+		yAxis = [OHGameControllerAxis
+		    oh_elementWithName: @"Left Thumbstick Y"
+				analog: true];
 		directionalPad = [OHGameControllerDirectionalPad
-		    oh_padWithName: @"Right Thumbstick"
+		    oh_padWithName: @"Left Thumbstick"
 			     xAxis: xAxis
 			     yAxis: yAxis
 			    analog: true];
 
 		_directionalPads = [[OFDictionary alloc]
 		    initWithObject: directionalPad
-			    forKey: @"Right Thumbstick"];
+			    forKey: @"Left Thumbstick"];
 
 		objc_autoreleasePoolPop(pool);
 	} @catch (id e) {
@@ -129,7 +173,7 @@ static const size_t numButtons = sizeof(buttonNames) / sizeof(*buttonNames);
 		name = @"ZR";
 		break;
 	case BTN_THUMBR:
-		name = @"Right Thumbstick";
+		name = @"Left Thumbstick";
 		break;
 	case BTN_START:
 		name = @"+";
@@ -154,14 +198,31 @@ static const size_t numButtons = sizeof(buttonNames) / sizeof(*buttonNames);
 {
 	switch (axis) {
 	case ABS_RX:
-		return [[_directionalPads
-		    objectForKey: @"Right Thumbstick"] xAxis];
+		return [[_directionalPads objectForKey: @"Left Thumbstick"]
+		    yAxis];
 	case ABS_RY:
-		return [[_directionalPads
-		    objectForKey: @"Right Thumbstick"] yAxis];
+		return [[_directionalPads objectForKey: @"Left Thumbstick"]
+		    xAxis];
 	}
 
 	return nil;
 }
 #endif
+
+#ifdef OF_HAVE_GCF
+- (OFDictionary<OFString *, NSString *> *)oh_buttonsMap
+{
+	return buttonsMap;
+}
+
+- (OFDictionary<OFString *, NSString *> *)oh_axesMap
+{
+	return [OFDictionary dictionary];
+}
+
+- (OFDictionary<OFString *, NSString *> *)oh_directionalPadsMap
+{
+	return directionalPadsMap;
+}
+#endif
 @end
