@@ -21,26 +21,56 @@
 
 #include <errno.h>
 
+#import <Foundation/Foundation.h>
+
 #import "OFSecureTransportTLSStream.h"
+#import "OFArray.h"
+#import "OFSecureTransportKeychain.h"
+#import "OFSecureTransportX509Certificate.h"
+
+#include <Security/SecCertificate.h>
+#include <Security/SecIdentity.h>
 
 #import "OFAlreadyOpenException.h"
+#import "OFInvalidArgumentException.h"
 #import "OFNotOpenException.h"
 #import "OFReadFailedException.h"
 #import "OFTLSHandshakeFailedException.h"
 #import "OFWriteFailedException.h"
 
+/*
+ * Apple deprecated Secure Transport without providing a replacement that can
+ * work with any socket. On top of that, their replacement, Network.framework,
+ * doesn't support STARTTLS at all.
+ */
+#ifdef __GNUC__
+# pragma GCC diagnostic ignored "-Wdeprecated"
+#endif
+
 int _ObjFWTLS_reference;
 
+static OFTLSStreamErrorCode
+statusToErrorCode(OSStatus status)
+{
+	switch (status) {
+	case errSSLXCertChainInvalid:
+		return OFTLSStreamErrorCodeCertificateVerificationFailed;
+	}
+
+	return OFTLSStreamErrorCodeUnknown;
+}
+
 static OSStatus
 readFunc(SSLConnectionRef connection, void *data, size_t *dataLength)
 {
+	OFStream *underlyingStream =
+	    ((OFTLSStream *)connection).underlyingStream;
 	bool incomplete;
 	size_t length;
 
 	@try {
-		length = [((OFTLSStream *)connection).underlyingStream
-		    readIntoBuffer: data
-			    length: *dataLength];
+		length = [underlyingStream readIntoBuffer: data
+						   length: *dataLength];
 	} @catch (OFReadFailedException *e) {
 		if (e.errNo == EWOULDBLOCK || e.errNo == EAGAIN) {
 			*dataLength = 0;
@@ -50,6 +80,9 @@ readFunc(SSLConnectionRef connection, void *data, size_t *dataLength)
 		@throw e;
 	}
 
+	if (length == 0 && underlyingStream.atEndOfStream)
+		return errSSLClosedAbort;
+
 	incomplete = (length < *dataLength);
 	*dataLength = length;
 
@@ -68,6 +101,8 @@ writeFunc(SSLConnectionRef connection, const void *data, size_t *dataLength)
 
 		if (e.errNo == EWOULDBLOCK || e.errNo == EAGAIN)
 			return errSSLWouldBlock;
+		if (e.errNo == EPIPE || e.errNo == ECONNRESET)
+			return errSSLClosedAbort;
 
 		@throw e;
 	}
@@ -75,15 +110,6 @@ writeFunc(SSLConnectionRef connection, const void *data, size_t *dataLength)
 	return noErr;
 }
 
-/*
- * Apple deprecated Secure Transport without providing a replacement that can
- * work with any socket. On top of that, their replacement, Network.framework,
- * doesn't support STARTTLS at all.
- */
-#if OF_GCC_VERSION >= 402
-# pragma GCC diagnostic ignored "-Wdeprecated"
-#endif
-
 @implementation OFSecureTransportTLSStream
 + (void)load
 {
@@ -124,9 +150,15 @@ writeFunc(SSLConnectionRef connection, const void *data, size_t *dataLength)
 	[_host release];
 	_host = nil;
 
-	SSLClose(_context);
+	@try {
+		SSLClose(_context);
+	} @catch (OFWriteFailedException *e) {
+		if (e.errNo != EPIPE)
+			@throw e;
+	}
+
 #ifdef HAVE_SSLCREATECONTEXT
-	CFRelease(_context);
+	[(id)_context release];
 #else
 	SSLDisposeContext(_context);
 #endif
@@ -181,8 +213,9 @@ writeFunc(SSLConnectionRef connection, const void *data, size_t *dataLength)
 	    bufferSize > 0));
 }
 
-- (void)asyncPerformClientHandshakeWithHost: (OFString *)host
-				runLoopMode: (OFRunLoopMode)runLoopMode
+- (void)of_asyncPerformHandshakeWithHost: (OFString *)host
+				  server: (bool)server
+			     runLoopMode: (OFRunLoopMode)runLoopMode
 {
 	static const OFTLSStreamErrorCode initFailedErrorCode =
 	    OFTLSStreamErrorCodeInitializationFailed;
@@ -194,10 +227,11 @@ writeFunc(SSLConnectionRef connection, const void *data, size_t *dataLength)
 		@throw [OFAlreadyOpenException exceptionWithObject: self];
 
 #ifdef HAVE_SSLCREATECONTEXT
-	if ((_context = SSLCreateContext(kCFAllocatorDefault, kSSLClientSide,
+	if ((_context = SSLCreateContext(kCFAllocatorDefault,
+	    (server ? kSSLServerSide : kSSLClientSide),
 	    kSSLStreamType)) == NULL)
 #else
-	if (SSLNewContext(false, &_context) != noErr)
+	if (SSLNewContext(server, &_context) != noErr)
 #endif
 		@throw [OFTLSHandshakeFailedException
 		    exceptionWithStream: self
@@ -212,8 +246,9 @@ writeFunc(SSLConnectionRef connection, const void *data, size_t *dataLength)
 			      errorCode: initFailedErrorCode];
 
 	_host = [host copy];
+	_server = server;
 
-	if (_verifiesCertificates)
+	if (!server && _verifiesCertificates)
 		if (SSLSetPeerDomainName(_context,
 		    _host.UTF8String, _host.UTF8StringLength) != noErr)
 			@throw [OFTLSHandshakeFailedException
@@ -221,6 +256,58 @@ writeFunc(SSLConnectionRef connection, const void *data, size_t *dataLength)
 					   host: _host
 				      errorCode: initFailedErrorCode];
 
+	if (_certificateChain.count > 0) {
+		bool first = true;
+		NSMutableArray *array;
+		SecCertificateRef firstCertificate;
+
+		array = [NSMutableArray
+		    arrayWithCapacity: _certificateChain.count];
+		if (array == nil)
+			@throw [OFTLSHandshakeFailedException
+			    exceptionWithStream: self
+					   host: _host
+				      errorCode: initFailedErrorCode];
+
+		firstCertificate = ((OFSecureTransportX509Certificate *)
+		    _certificateChain.firstObject).of_certificate;
+
+		if (CFGetTypeID(firstCertificate) == SecIdentityGetTypeID())
+			[array addObject: (id)firstCertificate];
+#ifndef OF_IOS
+		else {
+			SecKeychainRef keychain = [[OFSecureTransportKeychain
+			    temporaryKeychain] keychain];
+			SecIdentityRef identity;
+
+			if (SecIdentityCreateWithCertificate(keychain,
+			    firstCertificate, &identity) != noErr)
+				@throw [OFTLSHandshakeFailedException
+				    exceptionWithStream: self
+						   host: _host
+					      errorCode: initFailedErrorCode];
+
+			[array addObject: (id)identity];
+			[(id)identity release];
+		}
+#else
+		else
+			@throw [OFInvalidArgumentException exception];
+#endif
+
+		for (OFSecureTransportX509Certificate *certificate in
+		    _certificateChain) {
+			if (first) {
+				first = false;
+				continue;
+			}
+
+			[array addObject: (id)certificate.of_certificate];
+		}
+
+		SSLSetCertificate(_context, (CFArrayRef)array);
+	}
+
 	status = SSLHandshake(_context);
 
 	if (status == errSSLWouldBlock) {
@@ -245,17 +332,39 @@ writeFunc(SSLConnectionRef connection, const void *data, size_t *dataLength)
 		exception = [OFTLSHandshakeFailedException
 		    exceptionWithStream: self
 				   host: _host
-			      errorCode: OFTLSStreamErrorCodeUnknown];
+			      errorCode: statusToErrorCode(status)];
 
-	if ([_delegate respondsToSelector:
-	    @selector(stream:didPerformClientHandshakeWithHost:exception:)])
-		[_delegate		       stream: self
-		    didPerformClientHandshakeWithHost: _host
-					    exception: exception];
+	if (server) {
+		if ([(id <OFObject>)_delegate respondsToSelector: @selector(
+		    streamDidPerformServerHandshake:exception:)])
+			[_delegate streamDidPerformServerHandshake: self
+							 exception: exception];
+	} else {
+		if ([(id <OFObject>)_delegate respondsToSelector: @selector(
+		    stream:didPerformClientHandshakeWithHost:exception:)])
+			[_delegate		       stream: self
+			    didPerformClientHandshakeWithHost: _host
+						    exception: exception];
+	}
 
 	objc_autoreleasePoolPop(pool);
 }
 
+- (void)asyncPerformClientHandshakeWithHost: (OFString *)host
+				runLoopMode: (OFRunLoopMode)runLoopMode
+{
+	[self of_asyncPerformHandshakeWithHost: host
+					server: false
+				   runLoopMode: runLoopMode];
+}
+
+- (void)asyncPerformServerHandshakeWithRunLoopMode: (OFRunLoopMode)runLoopMode
+{
+	[self of_asyncPerformHandshakeWithHost: nil
+					server: true
+				   runLoopMode: runLoopMode];
+}
+
 -      (bool)stream: (OFStream *)stream
   didReadIntoBuffer: (void *)buffer
 	     length: (size_t)length
@@ -271,14 +380,21 @@ writeFunc(SSLConnectionRef connection, const void *data, size_t *dataLength)
 			exception = [OFTLSHandshakeFailedException
 			    exceptionWithStream: self
 					   host: _host
-				      errorCode: OFTLSStreamErrorCodeUnknown];
+				      errorCode: statusToErrorCode(status)];
 	}
 
-	if ([_delegate respondsToSelector:
-	    @selector(stream:didPerformClientHandshakeWithHost:exception:)])
-		[_delegate		       stream: self
-		    didPerformClientHandshakeWithHost: _host
-					    exception: exception];
+	if (_server) {
+		if ([(id <OFObject>)_delegate respondsToSelector: @selector(
+		    streamDidPerformServerHandshake:exception:)])
+			[_delegate streamDidPerformServerHandshake: self
+							 exception: exception];
+	} else {
+		if ([(id <OFObject>)_delegate respondsToSelector: @selector(
+		    stream:didPerformClientHandshakeWithHost:exception:)])
+			[_delegate		       stream: self
+			    didPerformClientHandshakeWithHost: _host
+						    exception: exception];
+	}
 
 	[_delegate release];
 
