@@ -22,6 +22,8 @@
 #include <errno.h>
 #include <string.h>
 
+#include "unistd_wrapper.h"
+
 #import "ObjFW.h"
 #import "ObjFWTest.h"
 
@@ -35,6 +37,11 @@
 	OFUNIXStreamSocket *sockClient, *sockServer, *sockAccepted;
 	char buffer[5];
 
+#ifdef OF_WINDOWS
+	if ([OFSystemInfo wineVersion] != nil)
+		OTSkip(@"UNIX stream sockets are broken on Wine");
+#endif
+
 #if defined(OF_HAVE_FILES) && !defined(OF_IOS)
 	path = [[OFSystemInfo temporaryDirectoryIRI]
 	    IRIByAppendingPathComponent: [[OFUUID UUID] UUIDString]]
@@ -68,6 +75,11 @@
 	}
 
 	@try {
+#ifndef OF_WINDOWS
+		OFUNIXSocketCredentials peerCredentials;
+		OFNumber *number;
+#endif
+
 		[sockServer listen];
 
 		[sockClient connectToPath: path];
@@ -80,6 +92,26 @@
 
 		OTAssertEqual(OFSocketAddressUNIXPath(
 		    sockAccepted.remoteAddress).length, 0);
+
+#ifndef OF_WINDOWS
+		peerCredentials = sockAccepted.peerCredentials;
+
+		number = [peerCredentials objectForKey:
+		    OFUNIXSocketCredentialsUserID];
+		if (number != nil)
+			OTAssertEqualObjects(number,
+			    [OFNumber numberWithUnsignedLong: getuid()]);
+		number = [peerCredentials objectForKey:
+		    OFUNIXSocketCredentialsGroupID];
+		if (number != nil)
+			OTAssertEqualObjects(number,
+			    [OFNumber numberWithUnsignedLong: getgid()]);
+		number = [peerCredentials objectForKey:
+		    OFUNIXSocketCredentialsProcessID];
+		if (number != nil)
+			OTAssertEqualObjects(number,
+			    [OFNumber numberWithUnsignedLong: getpid()]);
+#endif
 	} @finally {
 #ifdef OF_HAVE_FILES
 		[[OFFileManager defaultManager] removeItemAtPath: path];
